﻿/// <reference path="library/jquery-1.8.2.js" />
/// <reference path="library/jquery-ui-1.9.0.custom.js" />
/// <reference path="DWViewer.js" />
/// <reference path="DWViewerModel.js" />



// Keyboard handling
var DWKeyboardController = new function () {
    this._shortcuts = [];
    var self = this;

    this.keyData = function (keyCode, fn, ctrl, shift) {
        this.keyCode = keyCode;
        this.ctrl = ctrl;
        this.shift = shift;
        this.fn = fn;
    };

    this.addShortcut = function (key, fn, ctrl, shift) {
        if (key.toLowerCase) {  // If the key is string we assume a character.
            this._shortcuts[this._shortcuts.length] = new this.keyData(key.toUpperCase().charCodeAt(0), fn, ctrl, shift);
        }
        else
            this._shortcuts[this._shortcuts.length] = new this.keyData(key, fn, ctrl, shift);
    };

    this._keyDownHandler = function (e) {
        for (var idx in self._shortcuts) {
            var shortcut = self._shortcuts[idx];
            if (shortcut.keyCode == e.which) {
                var handled = false;
                if (e.ctrlKey) {
                    if (shortcut.ctrl) {
                        if (e.shiftKey) {
                            if (shortcut.shift) {
                                shortcut.fn();
                                handled = true;
                            }
                        }
                        else {
                            if (!shortcut.shift) {
                                shortcut.fn();
                                handled = true;
                            }
                        }
                    }
                }
                else {
                    if (!shortcut.ctrl) {
                        shortcut.fn();
                        handled = true;
                    }
                }
                if (handled) {  // Prevent the default action for the keyboard event if we have one of our own shortcuts used.
                    e.preventDefault();
                    e.stopPropagation();
                    return false;
                }
            }
        }

        return true;
    };

    this.attachToElement = function () {
        if (isNewWebClient())
            return $("#viewer");
        else
            return $(document);
    };

    this.startListening = function () {
        this.attachToElement().on("keydown.DWKeyboardController", this._keyDownHandler);
    }

    this.stopListening = function () {
        this.attachToElement().off("keydown.DWKeyboardController", this._keyDownHandler);
    };
};

DWKeyboardController.addShortcut($.ui.keyCode.PAGE_UP, function () {    // Go to previous page in current document
    var p = ViewerModel.pageNumber();
    if (p > 1) {
        ViewerModel.pageNumber(p - 1);
    }
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.PAGE_DOWN, function () {    // Go to next page in current document
    var p = ViewerModel.pageNumber();
    ViewerModel.hasMorePages()
    if (p < ViewerModel.maxPageValue() || ViewerModel.hasMorePages()) {
        ViewerModel.pageNumber(p + 1);
    }
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.LEFT, function () {    // Go to previous Document
    var d = ViewerModel.docNumber();
    if (d > 1) {
        ViewerModel.docNumber(d - 1);
    }
}, true, false);

DWKeyboardController.addShortcut($.ui.keyCode.LEFT, function () {    // Go to previous file in document
    var f = ViewerModel.fileNumber();
    if (f > 1) {
        ViewerModel.fileNumber(f - 1);
    }
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.RIGHT, function () {    // Go to next Document
    var d = ViewerModel.docNumber();
    if (d < ViewerModel.maxDocValue()) {
        ViewerModel.docNumber(d + 1);
    }
}, true, false);

DWKeyboardController.addShortcut($.ui.keyCode.RIGHT, function () {    // Go to next file in document
    var f = ViewerModel.fileNumber();
    if (f < ViewerModel.maxFileValue()) {
        ViewerModel.fileNumber(f + 1);
    }
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.UP, function () {    // Scroll up in current page
    var s = Viewer.element.scrollTop();
    Viewer.element.scrollTop(s - 100);
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.DOWN, function () {    // Scroll down in current page
    var s = Viewer.element.scrollTop();
    Viewer.element.scrollTop(s + 100);
}, false, false);

//DWKeyboardController.addShortcut("e", function () {    // Edit document
//    ViewerModel.editDocument();
//}, true, false);

DWKeyboardController.addShortcut("y", function () {    // Print document with annotations
    if (ViewerModel.UserRights.PrintDownloadSendEnable())
        window.open(PlatformAPI.downloadCurrentDocumentInPdf(true, false), 'Download');
}, true, false);

DWKeyboardController.addShortcut("m", function () {    // Email document in original format
    //ViewerModel.printClicked(ViewerModel.tools_email_context, null);
    if (ViewerModel.UserRights.PrintDownloadSendEnable())
        ViewerModel.emailDocument(SendDocumentOptions.Document_Original);
}, true, false);

DWKeyboardController.addShortcut("d", function () {    // Download document in original format
    if (ViewerModel.UserRights.PrintDownloadSendEnable())
        window.open(PlatformAPI.downloadCurrentDocument(), 'Download');
}, true, false);

DWKeyboardController.addShortcut("i", function () {    // Open info dialog
    ViewerModel.showDocumentInformation();
}, true, false);

DWKeyboardController.addShortcut("t", function () {    // Activate/Deactivate thumbnails view
    if (Viewer.isInDocumentOverview())
        Viewer.hideDocumentOverview();
    else
        Viewer.showDocumentOverview();
}, true, false);

DWKeyboardController.addShortcut("f", function () {    // Full-text search
    ViewerModel.fulltextSearchClicked();
}, true, false);

DWKeyboardController.addShortcut("f", function () {    // Proceed with full-text search
    ViewerModel.fulltextSearchNext();
}, true, true);

//DWKeyboardController.addShortcut("q", function () {    // Switch view
//    var newMode;
//    switch (ViewerModel.zoomMode) {
//        case ZoomMode.WholeDocument:
//            newMode = ZoomMode.FitToText;
//            break;
//        case ZoomMode.FitToText:
//            newMode = ZoomMode.FitToWidth;
//            break;
//        case ZoomMode.FitToWidth:
//            newMode = ZoomMode.WholeDocument;
//            break;
//        default:
//            newMode = ZoomMode.FitToText;
//            break;
//    }

//    ViewerModel.changeDisplayMode(newMode);
//}, true, false);

DWKeyboardController.addShortcut("s", function () {    // Separate Document
    ViewerModel.splitDocument();
}, true, false);

DWKeyboardController.addShortcut("r", function () {    // Rotate 90° to the right
    ViewerModel.Display_RotateRight();
}, true, false);

DWKeyboardController.addShortcut($.ui.keyCode.ESCAPE, function () {    // Reset to the default pan viewer tool
    ViewerModel.changeViewerMode(ViewerMode.Pan);
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.NUMPAD_ADD, function () {    // ZoomIn
    ViewerModel.Display_ZoomIn();
}, false, false);

DWKeyboardController.addShortcut($.ui.keyCode.NUMPAD_SUBTRACT, function () {    // ZoomOut
    ViewerModel.Display_ZoomOut();
}, false, false);